@@ -33,7 +33,7 @@
 GtkHTMLClassProperties *
 gtk_html_class_properties_new (void)
 {
-	GtkHTMLClassProperties *p = g_new (GtkHTMLClassProperties, 1);
+	GtkHTMLClassProperties *p = g_new0 (GtkHTMLClassProperties, 1);
 
 	/* default values */
 	p->magic_links             = TRUE;
@@ -190,7 +190,7 @@ gtk_html_class_properties_load (GtkHTMLClassProperties *p)
 	g_free (s);
 
 	GET  (bool, live_spell_check, "live_spell_check=true");
-	GET  (int, spell_error_color.red,   "spell_error_color_red=0xffff");
+	GET  (int, spell_error_color.red,   "spell_error_color_red=65535");
 	GET  (int, spell_error_color.green, "spell_error_color_green=0");
 	GET  (int, spell_error_color.blue,  "spell_error_color_blue=0");
 	GETS (language, "language=en");
@@ -284,8 +284,8 @@ static GtkEnumValue _gtk_html_command_values[] = {
   { GTK_HTML_COMMAND_INSERT_RULE, "GTK_HTML_COMMAND_INSERT_RULE", "insert-rule" },
   { GTK_HTML_COMMAND_INSERT_PARAGRAPH, "GTK_HTML_COMMAND_INSERT_PARAGRAPH", "insert-paragraph" },
   { GTK_HTML_COMMAND_INSERT_TAB, "GTK_HTML_COMMAND_INSERT_TAB", "insert-tab" },
-  { GTK_HTML_COMMAND_INSERT_TAB_OR_INDENT_MORE_OR_NEXT_CELL,
-    "GTK_HTML_COMMAND_INSERT_TAB_OR_INDENT_MORE_OR_NEXT_CELL", "insert-tab-or-indent-more-or-next-cell" },
+  { GTK_HTML_COMMAND_INSERT_TAB_OR_NEXT_CELL,
+    "GTK_HTML_COMMAND_INSERT_TAB_OR_NEXT_CELL", "insert-tab-or-next-cell" },
   { GTK_HTML_COMMAND_DELETE, "GTK_HTML_COMMAND_DELETE", "delete" },
   { GTK_HTML_COMMAND_DELETE_BACK, "GTK_HTML_COMMAND_DELETE_BACK", "delete-back" },
   { GTK_HTML_COMMAND_DELETE_BACK_OR_INDENT_DEC, "GTK_HTML_COMMAND_DELETE_BACK_OR_INDENT_DEC", "delete-back-or-indent-dec" },
@@ -319,7 +319,7 @@ static GtkEnumValue _gtk_html_command_values[] = {
   { GTK_HTML_COMMAND_INDENT_INC, "GTK_HTML_COMMAND_INDENT_INC", "indent-more" },
   { GTK_HTML_COMMAND_INDENT_INC_OR_NEXT_CELL, "GTK_HTML_COMMAND_INDENT_INC_OR_NEXT_CELL", "indent-more-or-next-cell" },
   { GTK_HTML_COMMAND_INDENT_DEC, "GTK_HTML_COMMAND_INDENT_DEC", "indent-less" },
-  { GTK_HTML_COMMAND_INDENT_DEC_OR_PREV_CELL, "GTK_HTML_COMMAND_INDENT_DEC_OR_PREV_CELL", "indent-less-or-prev-cell" },
+  { GTK_HTML_COMMAND_PREV_CELL, "GTK_HTML_COMMAND_PREV_CELL", "prev-cell" },
   { GTK_HTML_COMMAND_INDENT_PARAGRAPH, "GTK_HTML_COMMAND_INDENT_PARAGRAPH", "indent-paragraph" },
   { GTK_HTML_COMMAND_BREAK_AND_FILL_LINE, "GTK_HTML_COMMAND_BREAK_AND_FILL_LINE", "break-and-fill" },
   { GTK_HTML_COMMAND_SPACE_AND_FILL_LINE, "GTK_HTML_COMMAND_SPACE_AND_FILL_LINE", "space-and-fill" },
