@@ -632,6 +632,18 @@ calc_size (HTMLObject *o,
 			runWidth = 0;
 			run = obj;
 			
+			if (lmargin >= rmargin) {
+				gint new_y;
+
+				html_object_calc_size (run, painter);
+				html_clue_find_free_area (HTML_CLUE (o->parent), o->y, run->min_width,
+							  run->ascent + run->descent, indent, &new_y,
+							  &lmargin, &rmargin);
+				o->ascent += new_y - o->y;
+				o->y       = new_y;
+				w          = lmargin;
+			}
+
 			while ( run
 				&& ! (run->flags & HTML_OBJECT_FLAG_SEPARATOR)
 				&& ! (run->flags & HTML_OBJECT_FLAG_NEWLINE)
@@ -748,14 +760,15 @@ calc_size (HTMLObject *o,
 					w = set_line_x (&obj, run, w, &changed);
 					/* we've used up this line so insert a newline */
 					newLine = TRUE;
-				}
-				lmargin = html_object_get_left_margin (o->parent, painter, o->y);
-				
-				if (indent > lmargin)
-					lmargin = indent;
 
-				/* rmargin = html_object_get_right_margin (o->parent, painter, o->y); */
-				rmargin = pref_right_margin (painter, HTML_CLUEFLOW (o), o->parent, o->y);
+					lmargin = html_object_get_left_margin (o->parent, painter, o->y);
+				
+					if (indent > lmargin)
+						lmargin = indent;
+
+				        /* rmargin = html_object_get_right_margin (o->parent, painter, o->y); */
+					rmargin = pref_right_margin (painter, HTML_CLUEFLOW (o), o->parent, o->y);
+				}
 			}
 		}
 		
@@ -769,7 +782,7 @@ calc_size (HTMLObject *o,
 			halign = html_clueflow_get_halignment (flow);
 
 			if (w > o->width)
-				o->width = MIN (w, o->max_width);
+				o->width = w; // MIN (w, o->max_width);
 
 			if (halign == HTML_HALIGN_CENTER) {
 				extra = (rmargin - w) / 2;
@@ -1031,10 +1044,10 @@ draw_item (HTMLObject *self, HTMLPainter *painter, gint x, gint y, gint width, g
 
 		number = get_item_number_str (flow);
 		if (number) {
-			gint width, len;
+			gint width, len, line_offset = 0;
 
 			len   = strlen (number);
-			width = html_painter_calc_text_width (painter, number, len, 0,
+			width = html_painter_calc_text_width (painter, number, len, &line_offset,
 							      html_clueflow_get_default_font_style (flow), NULL)
 				+ html_painter_get_space_width (painter, html_clueflow_get_default_font_style (flow), NULL);
 			html_painter_set_font_style (painter, html_clueflow_get_default_font_style (flow));
@@ -1146,7 +1159,7 @@ check_point (HTMLObject *self,
 
 	/* before */
 	p = clue->head;
-	if (html_object_is_text (p))
+	if (p && html_object_is_text (p))
 		p = p->next;
 	if (p && (x < p->x || y < p->y - p->ascent)) {
 		obj = html_object_check_point (p, painter, p->x, p->y - p->ascent, offset_return, for_cursor);
@@ -1172,9 +1185,10 @@ check_point (HTMLObject *self,
 				return obj;
 		}
 	}
+
 	/* after */
 	p = clue->tail;
-	if (x >= p->x + p->width || y >= p->y + p->descent) {
+	if (p && ((x >= p->x + p->width) || (y >= p->y + p->descent))) {
 		obj = html_object_check_point (p, painter, MAX (0, p->x + p->width - 1), p->y + p->descent - 1,
 					       offset_return, for_cursor);
 		if (obj != NULL)
@@ -1394,26 +1408,26 @@ get_start_tag (HTMLObject *self)
 		case HTML_LIST_TYPE_UNORDERED:
 		case HTML_LIST_TYPE_MENU:
 		case HTML_LIST_TYPE_DIR:
-			return g_strdup (need_list_begin (self) ? "ul><li" : "li");
+			return g_strdup (need_list_begin (self) ? "ul>\n<li" : "li");
 		case HTML_LIST_TYPE_ORDERED_ARABIC:
 			return need_list_begin (self)
-				? g_strdup ("ol type=1><li")
+				? g_strdup ("ol type=1>\n<li")
 				: g_strdup_printf ("li type=1 value=%d", HTML_CLUEFLOW (self)->item_number);
 		case HTML_LIST_TYPE_ORDERED_UPPER_ROMAN:
 			return need_list_begin (self)
-				? g_strdup ("ol type=I><li")
+				? g_strdup ("ol type=I>\n<li")
 				: g_strdup_printf ("li type=I value=%d", HTML_CLUEFLOW (self)->item_number);
 		case HTML_LIST_TYPE_ORDERED_LOWER_ROMAN:
 			return need_list_begin (self)
-				? g_strdup ("ol type=i><li")
+				? g_strdup ("ol type=i>\n<li")
 				: g_strdup_printf ("li type=i value=%d", HTML_CLUEFLOW (self)->item_number);
 		case HTML_LIST_TYPE_ORDERED_UPPER_ALPHA:
 			return need_list_begin (self)
-				? g_strdup ("ol type=A><li")
+				? g_strdup ("ol type=A>\n<li")
 				: g_strdup_printf ("li type=A value=%d", HTML_CLUEFLOW (self)->item_number);
 		case HTML_LIST_TYPE_ORDERED_LOWER_ALPHA:
 			return need_list_begin (self)
-				? g_strdup ("ol type=a><li")
+				? g_strdup ("ol type=a>\n<li")
 				: g_strdup_printf ("li type=a value=%d", HTML_CLUEFLOW (self)->item_number);
 		}
 	case HTML_CLUEFLOW_STYLE_NORMAL:
@@ -1447,9 +1461,9 @@ get_end_tag (HTMLObject *self)
 		case HTML_LIST_TYPE_UNORDERED:
 		case HTML_LIST_TYPE_MENU:
 		case HTML_LIST_TYPE_DIR:
-			return g_strdup (need_list_begin (self) ? "ul" : "li");
+			return g_strdup (need_list_end (self) ? "li>\n</ul" : "li");
 		default:
-			return g_strdup (need_list_end (self) ? "ol" : "li");
+			return g_strdup (need_list_end (self) ? "li>\n</ol" : "li");
 		}
 	default:
 		return NULL;
@@ -2198,8 +2212,6 @@ html_clueflow_set_style (HTMLClueFlow *flow,
 	flow->style = style;
 	if (style != HTML_CLUEFLOW_STYLE_LIST_ITEM)
 		flow->item_number = 0;
-	else
-		flow->level = MAX (1, flow->level);
 
 	html_engine_schedule_update (engine);
 	/* FIXME - make it more effective: relayout_with_siblings (flow, engine); */
